@@ -320,17 +320,15 @@ dist_learn2 <- function(ylist, xlist, timelist, prior_init, tpm_init,
   }else if(totalgroup==1){#some common, some subject-specific
     
     tempcluster <- vector(mode="list",length=totalgroup)
-    olddiff <- 0
+   
     #initial value
     
     tempcommon <- J[jcommonindex,commonindex2]%*%t(parm[,commonindex2])
     z[commonindex] <- rowMeans(tempcommon) + colMeans(l[,commonindex])/rho
     
     #primal residual
-    olddiff <- olddiff + sum((tempcommon-z[commonindex])^2)
-    oldnorm <- sum(z^2)
-    olddiff <- olddiff / (1+oldnorm)
-    
+    olddiff <- sum((tempcommon-z[commonindex])^2)
+    oldznorm <- sum(z^2)
     olddualdiff <- 0
     olddualparm <- l
     olddualnorm <- 0
@@ -365,7 +363,7 @@ dist_learn2 <- function(ylist, xlist, timelist, prior_init, tpm_init,
     primal_change <- NULL
     dual_change <- NULL
     nllk_change <- NULL
-    
+    z_change <- NULL
     #recursion
     while(iteration<=maxit){
       newrho <- rho
@@ -454,13 +452,12 @@ dist_learn2 <- function(ylist, xlist, timelist, prior_init, tpm_init,
       }))
       
       #update z
-      newdiff <- 0
-      
+     
       tempcommon <- J[jcommonindex,commonindex2]%*%t(parm[,commonindex2])
       z[commonindex] <- rowMeans(tempcommon) + colMeans(l[,commonindex])/rho
-      newdiff <- newdiff + sum((tempcommon-z[commonindex])^2)
+      newdiff <- sum((tempcommon-z[commonindex])^2)
       newnorm <- sum(z^2)
-      relchange <- newdiff / (1+oldnorm)
+      relchange <- newdiff / (1+olddiff)
       primal_change <- c(primal_change,relchange)
       
       #update l
@@ -476,34 +473,38 @@ dist_learn2 <- function(ylist, xlist, timelist, prior_init, tpm_init,
       newdualdiff <- sum((newdualparm - olddualparm)^2)
       reldualchange <- newdualdiff / (olddualnorm +1)
       dual_change <- c(dual_change, reldualchange)
+      newznorm <- sum(z^2)
+      newzdiff <- abs(newznorm-oldznorm) / (1+oldznorm)
+      z_change <- c(z_change, newzdiff)
       
       if(iteration<=1) likbase <- nllk
       new_nllk_change <- abs(nllk-oldlik)/(1+oldlik)
       nllk_change <- c(nllk_change,new_nllk_change)
       
+      kkt_cur <- max(newzdiff, reldualchange, new_nllk_change)
       if(iteration > maxit | 
-         (iteration>2 & relchange < tol & reldualchange < tol )) {
+         (iteration>2 & kkt_cur < tol )) {
         nllk <- oldlik; break}
       
       if(print==TRUE & iteration>=2){
-        cat("iter:",iteration, "; residual_rel_chg:", relchange,
-            "; dual_rel_chg:",reldualchange,"; nllk_rel_chg:",new_nllk_change,"\n")
+        cat("iter:",iteration, "; kkt_residual:", kkt_cur,"\n")
       }
       
+      
       olddiff <- newdiff #
-      #oldparm <- newparm #
-      oldnorm <- newnorm #
+
       olddualdiff <- newdualdiff
       olddualparm <- newdualparm
       olddualnorm <- newdualnorm
-      
+      oldznorm <- newznorm
       old_nllk_change <- new_nllk_change
       iteration <- iteration + 1
     }
     #reorder back
     workingparm <- t(sapply(1:nrow(parm),function(kkk) invmapf(parm[kkk,],M,ncolx)))
-    return(list(working_parm=workingparm,common_parm=z,
-                residual_change=primal_change[-1],dual_change=dual_change[-1],
+    return(list(working_parm=workingparm,
+                common_parm=z,
+                kkt=pmax(z_change[-1],dual_change[-1],nllk_change[-1]),
                 nllk=nllk))
     #################
     
@@ -525,8 +526,7 @@ dist_learn2 <- function(ylist, xlist, timelist, prior_init, tpm_init,
     z[commonindex] <- rowMeans(tempcommon) + colMeans(l[,commonindex])/rho
     #primal residual
     olddiff <- olddiff + sum((tempcommon-z[commonindex])^2)
-    oldnorm <- sum(z^2)
-    olddiff <- olddiff / (1+oldnorm)
+    oldznorm <- sum(z^2)
     
     olddualdiff <- 0
     olddualparm <- l
@@ -562,6 +562,7 @@ dist_learn2 <- function(ylist, xlist, timelist, prior_init, tpm_init,
     primal_change <- NULL
     dual_change <- NULL
     nllk_change <- NULL
+    z_change <- NULL
     
     #recursion
     while(iteration<=maxit){
@@ -666,8 +667,11 @@ dist_learn2 <- function(ylist, xlist, timelist, prior_init, tpm_init,
       tempcommon <- J[jcommonindex,commonindex2]%*%t(parm[,commonindex2])
       z[commonindex] <- rowMeans(tempcommon) + colMeans(l[,commonindex])/rho
       newdiff <- newdiff + sum((tempcommon-z[commonindex])^2)
-      newnorm <- sum(z^2)
-      relchange <- newdiff / (1+oldnorm)
+      
+      newznorm <- sum(z^2)
+      newzdiff <- abs(newznorm - oldznorm) / (1+oldznorm)
+      z_change <- c(z_change, newzdiff)
+      relchange <- newdiff / (1+olddiff)
       primal_change <- c(primal_change,relchange)
       
       #update l
@@ -689,30 +693,29 @@ dist_learn2 <- function(ylist, xlist, timelist, prior_init, tpm_init,
       new_nllk_change <- abs(nllk-oldlik)/(1+oldlik)
       nllk_change <- c(nllk_change,new_nllk_change)
       
+      kkt_cur <- max(newzdiff, reldualchange, new_nllk_change)
       if(iteration > maxit | 
-         (iteration>2 & relchange < tol & reldualchange < tol )) {
+         (iteration>2 & kkt_cur < tol )) {
         nllk <- oldlik; break}
       
-      kkt_cur <- max(reldualchange, new_nllk_change)
       if(print==TRUE & iteration>=2){
         cat("iter:",iteration, "; kkt_residual:", kkt_cur,"\n")
       }
       
       olddiff <- newdiff #
-      #oldparm <- newparm #
-      oldnorm <- newnorm #
+     
       olddualdiff <- newdualdiff
       olddualparm <- newdualparm
       olddualnorm <- newdualnorm
-      
+      oldznorm <- newznorm
       old_nllk_change <- new_nllk_change
       iteration <- iteration + 1
     }
     #reorder back
     workingparm <- t(sapply(1:nrow(parm),function(kkk) invmapf(parm[kkk,],M,ncolx)))
     return(list(working_parm=workingparm,
-                #common_parm=z,
-                kkt=pmax(dual_change[-1],nllk_change[-1]),
+                common_parm=z,
+                kkt=pmax(z_change[-1],dual_change[-1],nllk_change[-1]),
                 nllk=nllk))
   }
   
